#!/bin/bash
# vim: et sts=4 sw=4

set -euo pipefail

readonly firmware=/usr/share/xow/firmware/FW_ACC_00U.bin
force=0

usage() {
    echo
    echo "Usage: $(basename $0) [OPTIONS]"
    echo
    echo "Downloads the firmware required by XOW. Saved as $firmware"
    echo
    echo "OPTIONS"
    echo
    echo "  --force"
    echo "        Overwrite existing file"
    echo
    echo "  --help"
    echo "        This help screen"
    echo
}

parse_args() {
    while [[ $# -gt 0 ]]; do
        case "$1" in
            -h | --help ) usage; exit 0;;
            -f | --force ) shift; force=1;;
            * ) usage; exit 1;;
        esac
    done
}

check_firmware() {
    ! [[ -e $firmware ]] && return

    if (( force == 0 )); then
        echo "Firmware file $firmware already exists. Exiting."
        exit 0
    fi

    echo "Firmware file $firmware exists. Removing file."
    pkexec rm $firmware
}

disclaimer() {
    local -r url="https://www.microsoft.com/en-us/legal/intellectualproperty/copyright"
    local -r title="Microsoft Terms of Use"
    local -r text="Firmware is distributed under the Microsoft Terms of Use.\n
                    Do you accept the terms of the agreement?"

    xdg-open "$url"
    if hash kdialog &>/dev/null; then
        ! kdialog --title "$title" --yesno "$text" && exit 0
        echo
        return
    fi
    if hash zenity &>/dev/null; then
        ! zenity --question --title "$title" --text "$text" && exit 0
        echo
        return
    fi
    echo "Cannot create license agreement prompt. Exiting"
    exit 1
}

get_firmware() {
    local -r url="http://download.windowsupdate.com/c/msdownload/update/driver/drvs/2017/07/1cd6a87c-623f-4407-a52d-c31be49e925c_e19f60808bdcbfbd3c3df6be3e71ffc52e43261e.cab"
    local -r sum="48084d9fa53b9bb04358f3bb127b7495dc8f7bb0b3ca1437bd24ef2b6eabdf66"
    local tmp=`mktemp -d`

    # TODO: trap rmdir
    cd "$tmp" &> /dev/null

    curl -o driver.cab "$url"
    cabextract -F FW_ACC_00U.bin driver.cab
    rm driver.cab

    echo $sum FW_ACC_00U.bin | sha256sum -c

    local -r destdir=`dirname $firmware`
    ! [[ -e $destdir ]] && pkexec mkdir -p $destdir
    pkexec mv `pwd`/FW_ACC_00U.bin $destdir

    cd - &> /dev/null
    rmdir $tmp
}

main() {
    parse_args $@
    check_firmware
    disclaimer
    get_firmware
}

main $@
